program Project1;

{$mode objfpc}{$H+}

uses
  SysUtils, Classes,
  fpimage, fpcanvas, fpimgcanv,
  fpreadjpeg, fpwritejpeg,
  fpreadpng, fpwritepng,
  extinterpolation;

type
  TInterpolationClass = class of TFPCustomInterpolation;

procedure ScaleImage(SourceFile, DestFile: String; DestWidth, DestHeight: integer;
  AInterpolationClass: TInterpolationClass);
var
  DestImg, SourceImg: TFPCustomImage;
  DestCanvas: TFPCustomCanvas;
  R: TFPCustomImageReader;
  W: TFPCustomImageWriter;
  ext: String;
begin
  // Read image
  SourceImg := TFPMemoryImage.Create(0, 0);
  try
    ext := Lowercase(ExtractFileExt(sourceFile));
    if (ext = '.jpg') or (ext = '.jpeg') then
      R := TFPReaderJPEG.Create
    else
    if (ext = '.png') then
      R := TFPReaderPNG.Create
    else begin
      WriteLn(SourceFile, ': Image format not supported');
      Halt;
    end;
    SourceImg.LoadFromFile(SourceFile, R);
  finally
    R.Free;
  end;

  DestImg := TFPMemoryImage.Create(DestWidth, DestHeight);
  try
    DestCanvas := TFPImageCanvas.Create(DestImg);
    try
      // Interpolate
      DestCanvas.Interpolation := AInterpolationClass.Create;
      try
        DestCanvas.StretchDraw(0, 0, DestWidth, DestHeight, SourceImg);
      finally
        DestCanvas.Interpolation.Free;
      end;
    finally
      DestCanvas.Free;
    end;
    // Save interpolated image
    ext := Lowercase(ExtractFileExt(sourceFile));
    if (ext = '.jpg') or (ext = '.jpeg') then
      W := TFPWriterJPEG.Create
    else
    if (ext = '.png') then
      W := TFPWriterPNG.Create
    else begin
      WriteLn(DestFile, ': Destination image format not supported');
      Halt;
    end;
    DestImg.SaveToFile(DestFile, W);
  finally
    DestImg.Free;
  end;
end;

function PrepareDestFile(ASourceFile: String; Appendix: String): String;
var
  ext: String;
begin
  ext := ExtractFileExt(ASourceFile);
  Result := ChangeFileExt(ASourceFile, '') + '- ' + Appendix + ext;
end;

const
  SRC_FILE1 = 'cheetah.jpg';        // 1083 x 739
  SRC_FILE2 = 'tcolorbox_200.png';  // 48 x 48;

begin
  ScaleImage(SRC_FILE1, PrepareDestFile(SRC_FILE1, 'cubic'), 108, 74, TCubicInterpolation);
  ScaleImage(SRC_FILE1, PrepareDestFile(SRC_FILE1, 'lanczos'), 108, 74, TLanczosInterpolation);
  ScaleImage(SRC_FILE2, PrepareDestFile(SRC_FILE2, 'cubic'), 96, 96, TCubicInterpolation);
  ScaleImage(SRC_FILE2, PrepareDestFile(SRC_FILE2, 'lanczos'), 96, 96, TLanczosInterpolation);

end.

